package appplatform

import (
	"context"
	"fmt"
	"net/http"

	"github.com/hashicorp/go-azure-sdk/sdk/client"
	"github.com/hashicorp/go-azure-sdk/sdk/odata"
)

// Copyright (c) Microsoft Corporation. All rights reserved.
// Licensed under the MIT License. See NOTICE.txt in the project root for license information.

type JobExecutionsListOperationResponse struct {
	HttpResponse *http.Response
	OData        *odata.OData
	Model        *[]JobExecution
}

type JobExecutionsListCompleteResult struct {
	LatestHttpResponse *http.Response
	Items              []JobExecution
}

type JobExecutionsListCustomPager struct {
	NextLink *odata.Link `json:"nextLink"`
}

func (p *JobExecutionsListCustomPager) NextPageLink() *odata.Link {
	defer func() {
		p.NextLink = nil
	}()

	return p.NextLink
}

// JobExecutionsList ...
func (c AppPlatformClient) JobExecutionsList(ctx context.Context, id JobId) (result JobExecutionsListOperationResponse, err error) {
	opts := client.RequestOptions{
		ContentType: "application/json; charset=utf-8",
		ExpectedStatusCodes: []int{
			http.StatusOK,
		},
		HttpMethod: http.MethodGet,
		Pager:      &JobExecutionsListCustomPager{},
		Path:       fmt.Sprintf("%s/executions", id.ID()),
	}

	req, err := c.Client.NewRequest(ctx, opts)
	if err != nil {
		return
	}

	var resp *client.Response
	resp, err = req.ExecutePaged(ctx)
	if resp != nil {
		result.OData = resp.OData
		result.HttpResponse = resp.Response
	}
	if err != nil {
		return
	}

	var values struct {
		Values *[]JobExecution `json:"value"`
	}
	if err = resp.Unmarshal(&values); err != nil {
		return
	}

	result.Model = values.Values

	return
}

// JobExecutionsListComplete retrieves all the results into a single object
func (c AppPlatformClient) JobExecutionsListComplete(ctx context.Context, id JobId) (JobExecutionsListCompleteResult, error) {
	return c.JobExecutionsListCompleteMatchingPredicate(ctx, id, JobExecutionOperationPredicate{})
}

// JobExecutionsListCompleteMatchingPredicate retrieves all the results and then applies the predicate
func (c AppPlatformClient) JobExecutionsListCompleteMatchingPredicate(ctx context.Context, id JobId, predicate JobExecutionOperationPredicate) (result JobExecutionsListCompleteResult, err error) {
	items := make([]JobExecution, 0)

	resp, err := c.JobExecutionsList(ctx, id)
	if err != nil {
		result.LatestHttpResponse = resp.HttpResponse
		err = fmt.Errorf("loading results: %+v", err)
		return
	}
	if resp.Model != nil {
		for _, v := range *resp.Model {
			if predicate.Matches(v) {
				items = append(items, v)
			}
		}
	}

	result = JobExecutionsListCompleteResult{
		LatestHttpResponse: resp.HttpResponse,
		Items:              items,
	}
	return
}
