package arcsettings

import (
	"fmt"
	"strings"

	"github.com/hashicorp/go-azure-helpers/resourcemanager/recaser"
	"github.com/hashicorp/go-azure-helpers/resourcemanager/resourceids"
)

// Copyright (c) Microsoft Corporation. All rights reserved.
// Licensed under the MIT License. See NOTICE.txt in the project root for license information.

func init() {
	recaser.RegisterResourceId(&ArcSettingId{})
}

var _ resourceids.ResourceId = &ArcSettingId{}

// ArcSettingId is a struct representing the Resource ID for a Arc Setting
type ArcSettingId struct {
	SubscriptionId    string
	ResourceGroupName string
	ClusterName       string
	ArcSettingName    string
}

// NewArcSettingID returns a new ArcSettingId struct
func NewArcSettingID(subscriptionId string, resourceGroupName string, clusterName string, arcSettingName string) ArcSettingId {
	return ArcSettingId{
		SubscriptionId:    subscriptionId,
		ResourceGroupName: resourceGroupName,
		ClusterName:       clusterName,
		ArcSettingName:    arcSettingName,
	}
}

// ParseArcSettingID parses 'input' into a ArcSettingId
func ParseArcSettingID(input string) (*ArcSettingId, error) {
	parser := resourceids.NewParserFromResourceIdType(&ArcSettingId{})
	parsed, err := parser.Parse(input, false)
	if err != nil {
		return nil, fmt.Errorf("parsing %q: %+v", input, err)
	}

	id := ArcSettingId{}
	if err = id.FromParseResult(*parsed); err != nil {
		return nil, err
	}

	return &id, nil
}

// ParseArcSettingIDInsensitively parses 'input' case-insensitively into a ArcSettingId
// note: this method should only be used for API response data and not user input
func ParseArcSettingIDInsensitively(input string) (*ArcSettingId, error) {
	parser := resourceids.NewParserFromResourceIdType(&ArcSettingId{})
	parsed, err := parser.Parse(input, true)
	if err != nil {
		return nil, fmt.Errorf("parsing %q: %+v", input, err)
	}

	id := ArcSettingId{}
	if err = id.FromParseResult(*parsed); err != nil {
		return nil, err
	}

	return &id, nil
}

func (id *ArcSettingId) FromParseResult(input resourceids.ParseResult) error {
	var ok bool

	if id.SubscriptionId, ok = input.Parsed["subscriptionId"]; !ok {
		return resourceids.NewSegmentNotSpecifiedError(id, "subscriptionId", input)
	}

	if id.ResourceGroupName, ok = input.Parsed["resourceGroupName"]; !ok {
		return resourceids.NewSegmentNotSpecifiedError(id, "resourceGroupName", input)
	}

	if id.ClusterName, ok = input.Parsed["clusterName"]; !ok {
		return resourceids.NewSegmentNotSpecifiedError(id, "clusterName", input)
	}

	if id.ArcSettingName, ok = input.Parsed["arcSettingName"]; !ok {
		return resourceids.NewSegmentNotSpecifiedError(id, "arcSettingName", input)
	}

	return nil
}

// ValidateArcSettingID checks that 'input' can be parsed as a Arc Setting ID
func ValidateArcSettingID(input interface{}, key string) (warnings []string, errors []error) {
	v, ok := input.(string)
	if !ok {
		errors = append(errors, fmt.Errorf("expected %q to be a string", key))
		return
	}

	if _, err := ParseArcSettingID(v); err != nil {
		errors = append(errors, err)
	}

	return
}

// ID returns the formatted Arc Setting ID
func (id ArcSettingId) ID() string {
	fmtString := "/subscriptions/%s/resourceGroups/%s/providers/Microsoft.AzureStackHCI/clusters/%s/arcSettings/%s"
	return fmt.Sprintf(fmtString, id.SubscriptionId, id.ResourceGroupName, id.ClusterName, id.ArcSettingName)
}

// Segments returns a slice of Resource ID Segments which comprise this Arc Setting ID
func (id ArcSettingId) Segments() []resourceids.Segment {
	return []resourceids.Segment{
		resourceids.StaticSegment("staticSubscriptions", "subscriptions", "subscriptions"),
		resourceids.SubscriptionIdSegment("subscriptionId", "12345678-1234-9876-4563-123456789012"),
		resourceids.StaticSegment("staticResourceGroups", "resourceGroups", "resourceGroups"),
		resourceids.ResourceGroupSegment("resourceGroupName", "example-resource-group"),
		resourceids.StaticSegment("staticProviders", "providers", "providers"),
		resourceids.ResourceProviderSegment("staticMicrosoftAzureStackHCI", "Microsoft.AzureStackHCI", "Microsoft.AzureStackHCI"),
		resourceids.StaticSegment("staticClusters", "clusters", "clusters"),
		resourceids.UserSpecifiedSegment("clusterName", "clusterName"),
		resourceids.StaticSegment("staticArcSettings", "arcSettings", "arcSettings"),
		resourceids.UserSpecifiedSegment("arcSettingName", "arcSettingName"),
	}
}

// String returns a human-readable description of this Arc Setting ID
func (id ArcSettingId) String() string {
	components := []string{
		fmt.Sprintf("Subscription: %q", id.SubscriptionId),
		fmt.Sprintf("Resource Group Name: %q", id.ResourceGroupName),
		fmt.Sprintf("Cluster Name: %q", id.ClusterName),
		fmt.Sprintf("Arc Setting Name: %q", id.ArcSettingName),
	}
	return fmt.Sprintf("Arc Setting (%s)", strings.Join(components, "\n"))
}
